/******************************************************************************
*  Filename:       serialIO.h
*  Revised:        $ $
*  Revision:       $ $
*
*  Description:    Help functions for handling queues
*
*  Copyright (C) 2015 Texas Instruments Incorporated - http://www.ti.com/
*
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*    Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
******************************************************************************/

//*****************************************************************************
//
//! \addtogroup rfqueue_api
//! @{
//
//*****************************************************************************

#ifndef PROCESS_COMMAND_H
#define PROCESS_COMMAND_H

/* Weighted checksum value for each command */
/* Use for integer representation of every command for easy comparison */
/* against input command string */
#if 1
#define ABSOLUTE_FREQUENCY                   2112
#define TX_REPEAT                            481
#define START_TX                             560
#define STOP_TX                              451
#define START_RX                             546
#define STOP_RX                              439
#define RADIO_STATE                          694
#define TX_POWER                             521
#define TX_DATARATE                          621
#define RX_DATARATE                          619
#define TX_BIT_SHAPING                       1029
#define FREQ_DIV                             383
#define TX_BITMODE                           535
#define RX_FILTER_BW                         834
#define CW                                   49
#define GET_RADIO_CONFIG                     1120
#define GET_RADIO_REGISTER                   1883
#define WRITE_REGISTER_FUNCTION              3258
#define RESET                                205
#define START_TX_FH                          699
#define START_RX_FH                          685

#else
typedef enum command {
	NONE                                 = 0,
	INVALID                              = 1,
    ABSOLUTE_FREQUENCY                   = 2112,
    TX_REPEAT                            = 481,
    START_TX                             = 560,
    STOP_TX                              = 451,
    START_RX                             = 546,
    STOP_RX                              = 439,
    RADIO_STATE                          = 694,
    TX_POWER                             = 521,
    TX_DATARATE                          = 621,
    RX_DATARATE                          = 619,
    TX_BIT_SHAPING                       = 1029,
    FREQ_DIV                             = 383,
    TX_BITMODE                           = 535,
    RX_FILTER_BW                         = 834,
    CW                                   = 49,
    GET_RADIO_CONFIG                     = 1120,
    GET_RADIO_REGISTER                   = 1883,
    WRITE_REGISTER_FUNCTION              = 3258,
    RESET                                = 205
};
#endif

enum rf_state  {
	TX_TURN_ON,
	TX_ON,
	TX_TURN_OFF,
	TX_OFF,
	TX_CW_ON,
	TX_CW_OFF,
	TX_FH_ON,
	RX_TURN_ON,
	RX_ON,
	RX_TURN_OFF,
	RX_OFF,
	RX_FH_ON,
};


//*****************************************************************************
//
// API Overview
//
//*****************************************************************************
// int8_t command_process();
// void   command_get();
// int8_t command_set();
// void   freqToString();
// void   set_absoluteFrequency();


//*****************************************************************************
//
// API Functions and prototypes
//
//*****************************************************************************

//*****************************************************************************
//
//! Process the command
//!
//
//*****************************************************************************
int8_t   command_process();
void     command_get();
int8_t   command_set();
void     freqToString();
void     set_absoluteFrequency();
uint16_t getNumPacket();
uint32_t getPacketInterval();
void     getDataRate();
void     getTxPower();
void     setTxPower();
void     getBitShaping();
void     getFreqDeviation();
void     getTxPacket();
void     getRxFilterBW();
void     getRadioConfig();
void     printResponse();

//*****************************************************************************
//
// Mark the end of the C bindings section for C++ compilers.
//
//*****************************************************************************
#ifdef __cplusplus
}
#endif

#endif //  __UART_H__

//*****************************************************************************
//
//! Close the Doxygen group.
//! @}
//
//*****************************************************************************
